'----------------------------------------------------------------------------------------------------
'
' PropCade_Controller - for the PropCade SBC, using Microchip MCP23S17 I/O expander
'
' Copyright August 22, 2010 by William Henning
'
' This code is released under the terms of the MIT license, which is attached below.
'
'----------------------------------------------------------------------------------------------------

{{
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}

CON

'----------------------------------------------------------------------------------------------------
' Constants defined for MCP23S17 use
'----------------------------------------------------------------------------------------------------

  _clkmode          = xtal1 + pll16x
  _xinfreq          = 6_250_000

  GP_RIGHT  = %00000001
  GP_LEFT   = %00000010
  GP_DOWN   = %00000100
  GP_UP     = %00001000
  GP_START  = %00010000
  GP_SELECT = %00100000
  GP_B      = %01000000
  GP_A      = %10000000


'----------------------------------------------------------------------------------------------------
' Objects needed for PropCade Controller
'----------------------------------------------------------------------------------------------------

OBJ

  pio: "MCP23S17_PropCade"      ' I/O expander cog

VAR

  long mailbox[4]

  word joy0

'----------------------------------------------------------------------------------------------------
' Main
'
'----------------------------------------------------------------------------------------------------

PUB start

  pio.start(@mailbox) ' start up pioCOG

PUB read
  return process

PUB process|nes_bits
  pio.wrpbyte(0,$01,$3F) ' set JS_SEL & LED to output
  nes_bits := 0
  pio.wrpbyte(0,$13,$80) ' read UP/DOWN/LEFT/RIGHT/B/C
  joy0 := !pio.rdpbyte(0,$12) ' Joy0
  if joy0 & 1
    nes_bits |= GP_UP
  if joy0 & 2
    nes_bits |= GP_DOWN
  if joy0 & 4
    nes_bits |= GP_LEFT
  if joy0 & 8
    nes_bits |= GP_RIGHT
  if joy0 & 16
    nes_bits |= GP_B
  if joy0 & 32
    nes_bits |= GP_SELECT
  pio.wrpbyte(0,$13,$00) ' read UP/DOWN/A/START
  joy0 := !pio.rdpbyte(0,$12) ' Joy0
  if joy0 & 16
    nes_bits |= GP_A
  if joy0 & 32
    nes_bits |= GP_START
  return nes_bits

